from __future__ import annotations

import logging
from time import perf_counter
from typing import Callable

from asgi_correlation_id.context import correlation_id
from fastapi import FastAPI
from fastapi import Request
from fastapi import Response
from starlette.middleware.base import BaseHTTPMiddleware

from artemis_common.models.server import LogRequestModel
from artemis_common.models.server import LogResponseModel


class AsyncIteratorWrapper:
    def __init__(self, obj):
        self._it = iter(obj)

    def __aiter__(self):
        return self

    async def __anext__(self):
        try:
            value = next(self._it)
        except StopIteration:
            raise StopAsyncIteration
        return value


class LoggingMiddleware(BaseHTTPMiddleware):
    def __init__(
        self,
        app: FastAPI,
        *,
        logger: logging.Logger,
    ) -> None:
        self._logger = logger
        super().__init__(app)

    async def dispatch(
        self,
        request: Request,
        call_next: Callable,
    ) -> Response:
        request_id = correlation_id.get()

        await self._log_request(request, request_id)

        start_time = perf_counter()
        response = await self._execute_request(request, call_next, request_id)
        end_time = perf_counter()

        process_time = round(end_time - start_time, 4)
        await self._log_response(response, request, request_id, process_time)

        return response

    async def _execute_request(
        self,
        request: Request,
        call_next: Callable,
        request_id: str,
    ) -> Response:
        try:
            response = await call_next(request)
            response.headers['X-API-Request-ID'] = request_id
            return response

        except Exception as ex:
            request_model = LogRequestModel(request=request)
            message = 'Failed to process response'
            self._logger.exception(
                message, extra=dict(
                    request_id=request_id,
                    request=request_model.dict(),
                    client_address=self._get_client_address(request),
                    exc_message=str(ex),
                ),
            )
            raise

    async def _log_request(
        self,
        request: Request,
        request_id: str,
    ):
        request_model = LogRequestModel(request=request)
        message = f'Request {request.method} {request.url}'
        self._logger.info(
            message, extra=dict(
                request_id=request_id,
                request=request_model.dict(),
                client_address=self._get_client_address(request),
            ),
        )

    async def _log_response(
        self,
        response: Response,
        request: Request,
        request_id: str,
        process_time: float,
    ):
        response_body = [section async for section in response.__dict__['body_iterator']]
        response.__setattr__(
            'body_iterator', AsyncIteratorWrapper(response_body),
        )

        response_length = 0
        for response_item in response_body:
            response_length += len(response_item)

        response_model = LogResponseModel(
            request=request,
            response=response,
            response_length=response_length,
        )
        message = f'Response {request.method} {request.url}'

        self._logger.info(
            message, extra=dict(
                request_id=request_id,
                client_address=self._get_client_address(request),
                response_time=process_time,
                response=response_model.dict(),
            ),
        )

    def _get_client_address(self, request: Request):
        client_address = request.headers.get('x-azure-clientip')
        if client_address:
            return client_address
        return request.headers.get('x-client-ip', request.client.host)



from __future__ import annotations

from datetime import datetime
from typing import Optional

from pydantic import Field
from pydantic import root_validator

from artemis_common.models.bases import AdvancedBaseModel
from artemis_common.models.bases import ValueBaseModel


class CategoryModel(AdvancedBaseModel):
    id: int
    name: str


class SerieModel(AdvancedBaseModel):
    id: str
    realtime_start: datetime
    realtime_end: datetime
    title: str
    observation_start: datetime
    observation_end: datetime
    frequency: str
    frequency_short: str
    units: str
    units_short: str
    seasonal_adjustment: str
    seasonal_adjustment_short: str
    last_updated: datetime
    last_updated_date: datetime
    popularity: int
    group_popularity: int
    notes: Optional[str]
    active: bool


class ObservationModel(AdvancedBaseModel):
    observations: list[ValueBaseModel]

    @root_validator(pre=True)
    def validate_values(cls, values):
        observations = values['observations']
        values['observations'] = [
            obs for obs in observations
            if obs['value'] != '.'
        ]
        return values


__all__ = [
    CategoryModel.__name__,
    SerieModel.__name__,
    ObservationModel.__name__,
]



from __future__ import annotations

from typing import Dict

from pydantic import BaseModel
from pydantic import Field
from pydantic import root_validator


def pythonize(value):
    result = ''
    last_capital = False
    for char in value:
        if char.isupper():
            if not last_capital:
                result += '_'
            result += char.lower()
            last_capital = True
        else:
            result += char
            last_capital = False
    return result


class Row(BaseModel):
    key: str
    new_key_name: str = Field(default='')
    value: float | int | str | None
    of_type: str = Field(default='')

    @root_validator(pre=True)
    @classmethod
    def before(cls, values):
        values['of_type'] = type(values['value']).__name__
        values['new_key_name'] = pythonize(values['key'])
        return values

    def __repr__(self):
        if self.key == self.new_key_name:
            return f'{self.key}: {self.of_type}'
        else:
            return f"{self.new_key_name}: {self.of_type} = Field(alias='{self.key}')"


def parse_to_model(obj: Dict):
    return [Row(key=k, value=v) for k, v in obj.items()]


def create_model(name: str, obj: Dict):
    rows = parse_to_model(obj)
    print(f'class {name}(BaseModel):')
    for row in rows:
        print(f'    {row.__repr__()}')



from __future__ import annotations

from typing import Iterable
from typing import Mapping
from typing import Optional
from typing import Type

import aiohttp
from httpx import Headers
from pydantic import BaseSettings

from .params_model import ParamsModel
from artemis_common import consts
from artemis_common.models.bases import AdvancedBaseModel
from artemis_common.models.server import ServerSuccessResponseModel


env_vars = consts._EnvironmentVariables()


def _get_dal_headers():
    return Headers(
        headers={
            consts.ARTEMIS_DAL_API_KEY_NAME: env_vars.artemis_dal_api_key.get_secret_value(),
            'Content-Type': 'application/json',
        },
    )


async def async_get(
    base_url: str,
    headers: Headers,
    route: str,
    params: Type[ParamsModel],
    output_model: Type[AdvancedBaseModel],
) -> Type[AdvancedBaseModel] | list[Type[AdvancedBaseModel]]:
    async with aiohttp.ClientSession(
        base_url=base_url,
        headers=headers,
    ) as client:
        raw_response = await client.get(
            url=route,
            params=params.get_params(),
            raise_for_status=True,
        )

    response = await raw_response.json()

    if isinstance(response, Iterable) and not isinstance(response, Mapping):
        return [output_model(**item) for item in response]
    return output_model(**response)


async def async_post(
    base_url: str,
    headers: Headers,
    route: str,
    data: Type[AdvancedBaseModel],
    params: Optional[Type[ParamsModel]] = None,
) -> ServerSuccessResponseModel:
    async with aiohttp.ClientSession(
        base_url=base_url,
        headers=headers,
    ) as client:
        raw_response = await client.post(
            url=route,
            params=params,
            json=data.dict(),
            raise_for_status=True,
        )

    response = await raw_response.json()
    return ServerSuccessResponseModel(**response)


async def dal_get(
    base_url: str,
    route: str,
    params: Type[ParamsModel],
    output_model: Type[AdvancedBaseModel],
) -> Type[AdvancedBaseModel] | list[Type[AdvancedBaseModel]]:
    return await async_get(
        base_url=base_url,
        headers=_get_dal_headers(),
        route=route,
        params=params,
        output_model=output_model,
    )


async def dal_post(
    base_url: str,
    route: str,
    data: Type[AdvancedBaseModel],
    params: Optional[Type[ParamsModel]] = None,
) -> ServerSuccessResponseModel:
    return await async_post(
        base_url=base_url,
        headers=_get_dal_headers(),
        route=route,
        data=data,
        params=params,
    )


__all__ = [
    dal_get.__name__,
    dal_post.__name__,
]



from __future__ import annotations

from fastapi import FastAPI
from fastapi import Request
from fastapi import status
from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse
from jwt.exceptions import ExpiredSignatureError
from jwt.exceptions import PyJWTError

from artemis_common import exceptions as artemis_exceptions
from artemis_common.models.server import ServerExceptionResponseModel


def add_base_exception_handlers(app: FastAPI):
    @app.exception_handler(artemis_exceptions.ArtemisOperationFailedError)
    async def handle_artemis_operation_failed_error(
        request: Request,
        exc: artemis_exceptions.ArtemisOperationFailedError,
    ):
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ServerExceptionResponseModel(
                message='Artemis server operation failed!',
                error=str(exc),
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(artemis_exceptions.ArtemisOperationUnexpectedlyFailedError)
    async def handle_artemis_operation_unexpectedly_failed_error(
        request: Request,
        exc: artemis_exceptions.ArtemisOperationUnexpectedlyFailedError,
    ):
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ServerExceptionResponseModel(
                message='[Unexpected] Artemis server operation failed unexpectedly!',
                error=str(exc),
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(artemis_exceptions.ArtemisUnexpectedFlowError)
    async def handle_artemis_unexpected_flow_reached_error(
        request: Request,
        exc: artemis_exceptions.ArtemisUnexpectedFlowError,
    ):
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ServerExceptionResponseModel(
                message='[Unexpected] This exception should never be raised!',
                error=str(exc),
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(ExpiredSignatureError)
    async def handle_expired_token_exception(
        request: Request,
        exc: ExpiredSignatureError,
    ):
        return JSONResponse(
            status_code=status.HTTP_401_UNAUTHORIZED,
            content=ServerExceptionResponseModel(
                message='Token expired!',
                error=str(exc),
                status_code=status.HTTP_401_UNAUTHORIZED,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(PyJWTError)
    async def handle_invalid_token_exception(
        request: Request,
        exc: PyJWTError,
    ):
        return JSONResponse(
            status_code=status.HTTP_401_UNAUTHORIZED,
            content=ServerExceptionResponseModel(
                message='Failed to decode token!',
                error=str(exc),
                status_code=status.HTTP_401_UNAUTHORIZED,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(artemis_exceptions.ArtemisException)
    async def handle_unclassified_artemis_exception(
        request: Request,
        exc: artemis_exceptions.ArtemisException,
    ):
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ServerExceptionResponseModel(
                message='[Unexpected] Artemis Unclassified Error',
                error=str(exc),
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(RequestValidationError)
    async def handle_request_validation_error(
        request: Request,
        exc: RequestValidationError,
    ):
        return JSONResponse(
            status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
            content=ServerExceptionResponseModel(
                message='Validation Error',
                error=str(exc),
                status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )

    @app.exception_handler(Exception)
    async def handle_unknown_exception(
        request: Request,
        exc: Exception,
    ):
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ServerExceptionResponseModel(
                message='[Unexpected] Server Error',
                error=str(exc),
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                url=str(request.url),
                path=request.url.path,
                params=request.query_params,
                exc=exc,
            ).dict(),
        )



from __future__ import annotations

from datetime import datetime

import jwt

from .config import read_pem
from .models import JWTDecodedPayloadModel


ALGORITHM = 'RS256'
PEM = read_pem()


def decode_token(token: str) -> JWTDecodedPayloadModel:
    if not token:
        raise jwt.exceptions.InvalidTokenError('Missing token!')
    raw_payload = jwt.decode(
        jwt=token,
        key=PEM,
        algorithms=ALGORITHM,
    )
    payload = JWTDecodedPayloadModel(**raw_payload)
    utc_now = datetime.utcnow()

    if utc_now >= payload.exp:
        raise jwt.exceptions.ExpiredSignatureError()

    return payload



from __future__ import annotations

from pydantic import BaseModel
from pydantic import Field


class ParamsModel(BaseModel):
    class Config:
        use_enum_values = True
        allow_population_by_field_name = True
        underscore_attrs_are_private = True


class DALParamsModel(ParamsModel):
    user_id: str = Field(default=None)
    category_id: int = Field(default=None, ge=0)
    gte: int = Field(default=None, ge=0, le=100)
    lte: int = Field(default=None, ge=0, le=100)
    serie_id: str = Field(default=None)

    def get_params(self):
        params_dict = self.dict(exclude_none=True, by_alias=True)
        return params_dict


__all__ = [
    DALParamsModel.__name__,
]



from __future__ import annotations

import json
import logging

from ddtrace import patch_all

from .config import CONFIG_DIR
from .config import get_config
from .config import read_pem

patch_all()

uvicorn_disable_logging_file = CONFIG_DIR.joinpath('uvicorn_disable_logging.json')
uvicorn_disable_logging_config = json.loads(uvicorn_disable_logging_file.read_text())
logging.config.dictConfig(uvicorn_disable_logging_config)



from __future__ import annotations

import logging
import platform
import sys
from typing import Callable
from typing import Optional

import structlog
from ddtrace import tracer
from ddtrace.runtime import RuntimeMetrics
from structlog.processors import CallsiteParameter
from structlog.types import EventDict
from structlog.types import Processor

from artemis_common.consts import artemis_env
from artemis_common.consts import is_artemis_debug
from artemis_common.consts import is_prod_env
from artemis_common.logging import RawTCPHandler
from artemis_common.models import LoggingCenterModel


def get_log_level(
    is_debug: bool = False,
) -> int:
    """
    is_debug: set log level to DEBUG
        this argument has the highest priority

    if `ARTEMIS_DEBUG` environment variable is true
        it will set log level to DEBUG

    Default log level: INFO
    """
    if is_debug:
        return logging.DEBUG
    if is_artemis_debug:
        return logging.DEBUG

    return logging.INFO


def is_use_console(
    is_prod: bool = False,
    is_console: bool = False,
) -> bool:
    """
    is_prod: set console logging to False
    is_console: set if use json logging or console logging

    Default is to use json logging
    """
    if is_prod:
        return False
    return is_console


def drop_color_message_key(_, __, event_dict: EventDict) -> EventDict:
    event_dict.pop('color_message', None)
    return event_dict


def tracer_injection(_, __, event_dict: EventDict) -> EventDict:
    # get correlation ids from current tracer context
    span = tracer.current_span()
    trace_id, span_id = (span.trace_id, span.span_id) if span else (None, None)

    dd = event_dict.get('dd', {})
    dd['trace_id'] = str(trace_id or 0)
    dd['span_id'] = str(span_id or 0)

    event_dict['dd'] = dd
    return event_dict


def datadog_injection(
    env: str,
    service: str,
    version: str,
) -> Callable:
    def service_injection_wrapper(_, __, event_dict: EventDict) -> EventDict:
        dd = event_dict.get('dd', {})
        dd['service'] = dd.get('service', service)
        dd['env'] = dd.get('env', env)
        dd['version'] = dd.get('version', version)

        event_dict['dd'] = dd
        return event_dict
    return service_injection_wrapper


def system_metadata_injection(_, __, event_dict: EventDict) -> EventDict:
    event_dict['host_name'] = event_dict.get('host_name', platform.node())
    event_dict['system'] = event_dict.get('system', platform.system())
    event_dict['platform'] = event_dict.get('platform', platform.platform())
    event_dict['python_version'] = event_dict.get('python_version', platform.python_version())
    return event_dict


def _reconfigure_uvicorn_logger():
    for _log in ['uvicorn', 'uvicorn.error']:
        # Clear the log handlers for uvicorn loggers, and enable propagation
        # so the messages are caught by our root logger and formatted correctly
        # by structlog
        logging.getLogger(_log).handlers.clear()
        logging.getLogger(_log).propagate = True

    # Since we re-create the access logs ourselves, to add all information
    # in the structured log (see the `logging_middleware` in main.py), we clear
    # the handlers and prevent the logs to propagate to a logger higher up in the
    # hierarchy (effectively rendering them silent).
    logging.getLogger('uvicorn.access').handlers.clear()
    logging.getLogger('uvicorn.access').propagate = False


def _handle_exception_hook(exc_type, exc_value, exc_traceback):
    """
    Log any uncaught exception instead of letting it be printed by Python
    (but leave KeyboardInterrupt untouched to allow users to Ctrl+C to stop)
    See https://stackoverflow.com/a/16993115/3641865
    """
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return

    root_logger = logging.getLogger()
    root_logger.error(
        'Uncaught exception', exc_info=(exc_type, exc_value, exc_traceback),
    )


def setup_logging(
    version: str,
    is_debug: bool = False,
    is_console: bool = False,
    logging_center: Optional[LoggingCenterModel] = None,
):
    log_level = get_log_level(is_debug=is_debug)
    use_console = is_use_console(is_prod=is_prod_env, is_console=is_console)
    service_name = None if not logging_center else logging_center.name

    shared_processors = [
        structlog.contextvars.merge_contextvars,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.add_log_level_number,
        structlog.processors.CallsiteParameterAdder([
            CallsiteParameter.PATHNAME,
            CallsiteParameter.FILENAME,
            CallsiteParameter.MODULE,
            CallsiteParameter.FUNC_NAME,
            CallsiteParameter.LINENO,
            CallsiteParameter.THREAD,
            CallsiteParameter.THREAD_NAME,
            CallsiteParameter.PROCESS,
            CallsiteParameter.PROCESS_NAME,
        ]),
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.stdlib.ExtraAdder(),
        drop_color_message_key,
        tracer_injection,
        datadog_injection(
            env=artemis_env,
            service=service_name,
            version=version,
        ),
        system_metadata_injection,
        structlog.processors.TimeStamper(fmt='iso', utc=True),
        structlog.processors.StackInfoRenderer(),
    ]

    if not use_console:
        shared_processors.append(structlog.processors.EventRenamer(to='message'))
        # Format the exception only for JSON logs, as we want to pretty-print them when
        # using the ConsoleRenderer instead of JSONRenderer
        shared_processors.append(structlog.processors.format_exc_info)

    structlog.configure(
        processors=shared_processors + \
        # Must be the last processor before calling `ProcessorFormatter`.
        [structlog.stdlib.ProcessorFormatter.wrap_for_formatter],
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )

    if use_console:
        log_renderer = structlog.dev.ConsoleRenderer()
    else:
        log_renderer = structlog.processors.JSONRenderer()

    formatter = structlog.stdlib.ProcessorFormatter(
        # These run ONLY on `logging` entries that do NOT originate within structlog.
        foreign_pre_chain=shared_processors,
        # These run on ALL entries after the pre_chain is done.
        processors=[
            structlog.stdlib.ProcessorFormatter.remove_processors_meta,
            log_renderer,
        ],
    )

    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    root_logger = logging.getLogger()
    root_logger.setLevel(log_level)
    root_logger.addHandler(stream_handler)

    if logging_center:
        tcp_handler = RawTCPHandler(
            host=logging_center.host,
            port=logging_center.logs_port,
        )
        tcp_handler.setFormatter(formatter)
        root_logger.addHandler(tcp_handler)

    _reconfigure_uvicorn_logger()

    sys.excepthook = _handle_exception_hook


def enable_runtime_metrics(logging_center: LoggingCenterModel):
    stats_url = f'{logging_center.host}:{logging_center.stats_port}'
    RuntimeMetrics.enable(tracer=tracer, dogstatsd_url=stats_url)



from __future__ import annotations

from datetime import datetime

from .bases import AdvancedBaseModel


class APIKeyModel(AdvancedBaseModel):
    api_key: str


class JWTModel(AdvancedBaseModel):
    token: str


class JWTPayloadModel(AdvancedBaseModel):
    user_id: str
    api_key_hash: str


class JWTDecodedPayloadModel(JWTPayloadModel):
    iss: str
    exp: datetime
    iat: datetime


__all__ = [
    APIKeyModel.__name__,
    JWTModel.__name__,
    JWTPayloadModel.__name__,
    JWTDecodedPayloadModel.__name__,
]



from __future__ import annotations

from fastapi import Request
from fastapi import Response
from pydantic import BaseModel


class LoggingCenterModel(BaseModel):
    name: str
    host: str
    logs_port: int
    stats_port: int


class LogRequestModel(BaseModel):
    method: str
    path: str
    query: str
    headers: dict

    def __init__(self, request: Request):
        super().__init__(
            method=request.method,
            path=request.url.path,
            query=request.url.query,
            headers=dict(request.headers),
        )


class LogResponseModel(BaseModel):
    length: int
    status_code: int

    def __init__(self, request: Request, response: Response, response_length: int):
        super().__init__(
            status_code=response.status_code,
            length=response_length,
        )


__all__ = [
    LoggingCenterModel.__name__,
    LogRequestModel.__name__,
    LogResponseModel.__name__,
]



from __future__ import annotations

from datetime import datetime
from typing import Optional

from pydantic import BaseModel
from pydantic import Field

from artemis_common.models.bases import AdvancedBaseModel


class UserAPIKeyModel(AdvancedBaseModel):
    id: str
    salt: str
    api_key_hash: str
    creation_time: str


__all__ = [
    UserAPIKeyModel.__name__,
]



from __future__ import annotations

from datetime import datetime
from typing import Optional

from dateutil.parser import parse
from pydantic import BaseModel
from pydantic import Extra
from pydantic import Field
from pydantic import root_validator


def parse_date_generic(date: str | datetime | int) -> datetime:
    if type(date) == datetime:
        return date.replace(tzinfo=None)

    if type(date) == int:
        return datetime.utcfromtimestamp(date)

    return parse(date).replace(tzinfo=None)


class AdvancedBaseModel(BaseModel):
    class Config:
        allow_population_by_field_name = True

    @root_validator(pre=True)
    def dates_parser(cls, values):
        scheme = cls.schema()
        properties = scheme['properties']
        for prop, settings in properties.items():
            if prop in values and \
                    settings.get('type') == 'string' and \
                    settings.get('format') == 'date-time':
                values[prop] = parse_date_generic(values[prop])
        return values


class DateBaseModel(AdvancedBaseModel):
    date: datetime


class ValueBaseModel(DateBaseModel):
    value: float


class CountBaseModel(AdvancedBaseModel):
    count: int



from __future__ import annotations

from pathlib import Path


def find_parent_file(folder: Path, filename: str) -> Path:
    if not folder.is_dir():
        raise NotADirectoryError(f'{folder=} is not a directory')
    if folder == folder.parent:
        raise FileNotFoundError(f'{filename=} is not found')
    for item in folder.iterdir():
        if item.name == filename:
            return item
    return find_parent_file(folder.parent, filename)



from __future__ import annotations

import json
import logging
from pathlib import Path
from typing import Type

from pydantic import BaseModel
from pydantic import ValidationError


class ConfigFileLoadError(Exception):
    pass


class ConfigLoader:
    def __init__(
        self,
        config_file_path: Path,
        config_model: Type[BaseModel],
    ) -> None:
        self._config_file_path = config_file_path
        self._config_model = config_model
        self._config = None

    @property
    def config(self) -> Type[BaseModel]:
        if self._config:
            return self._config
        self._config = self._load_config()
        self._setup_logging()
        return self._config

    def _load_config(self) -> Type[BaseModel]:
        try:
            raw_config = self._config_file_path.read_text()
            json_config = json.loads(raw_config)
            return self._config_model(**json_config)
        except OSError as ex:
            raise ConfigFileLoadError('Config file was not found!') from ex
        except ValidationError as ex:
            raise ConfigFileLoadError('Config file does not match the given model') from ex
        except ValueError as ex:
            raise ConfigFileLoadError('Config file is not a valid json!') from ex

    def _setup_logging(self):
        logging.config.dictConfig(self._config.logging)



from __future__ import annotations

from .bases import AdvancedBaseModel
from .bases import CountBaseModel
from .bases import DateBaseModel
from .bases import ValueBaseModel
from .jwt import *
from .server import LoggingCenterModel



from __future__ import annotations


class DALRoutes:
    API_kEY = '/v1/api_key'
    CATEGORIES_CHILDREN = '/v1/categories/children'
    SERIES_BY_CATEGORY = '/v1/series/by-category'
    SERIES_COUNT_BY_CATEGORY = '/v1/series/by-category/count'
    SERIES_BY_POPULARITY = '/v1/series/by-popularity'
    SERIES_COUNT_BY_POPULARITY = '/v1/series/by-popularity/count'
    SERIES_CATEGORIES_TREE = '/v1/series/categories-tree'


__all__ = [
    DALRoutes.__name__,
]



from __future__ import annotations

from logging.handlers import SocketHandler


class RawTCPHandler(SocketHandler):
    def emit(self, record):
        try:
            self.send((self.format(record)).encode())
        except Exception:
            self.handleError(record)



from __future__ import annotations

from pathlib import Path

import setuptools


PACKAGE_NAME = 'artemis-common'
PACKAGE_DIR = Path(__file__).resolve().parent


def read_requirements(filename: str) -> list[str]:
    requirements_file = PACKAGE_DIR.joinpath(filename)
    raw_requirements = requirements_file.read_text()
    return [line for line in raw_requirements.split('\n') if line]


def read_version():
    version_file = PACKAGE_DIR.joinpath('version')
    return version_file.read_text()


setuptools.setup(
    name=PACKAGE_NAME,
    version=read_version(),
    author='0xArti',
    author_email='contact@hunters-of-artemis.com',
    description='Artemis common project utilities',
    long_description_content_type='text/markdown',
    include_package_data=True,
    packages=setuptools.find_packages(),
    python_requires='>=3.10',
    install_requires=read_requirements('requirements.txt'),
)



from __future__ import annotations

from fastapi import FastAPI
from fastapi.openapi.docs import get_redoc_html
from fastapi.openapi.docs import get_swagger_ui_html


def add_swagger(app: FastAPI):
    @app.get('/redoc', include_in_schema=False)
    async def redoc():
        return get_redoc_html(
            openapi_url=app.openapi_url,
            title=f'{app.title} - Redoc',
            redoc_js_url='/static/redoc.standalone.js',
            redoc_favicon_url='/static/favicon-32x32.png',
        )

    @app.get('/docs', include_in_schema=False)
    async def docs():
        return get_swagger_ui_html(
            openapi_url=app.openapi_url,
            title=f'{app.title} - Swagger',
            swagger_js_url='/static/swagger-ui-bundle.js',
            swagger_css_url='/static/swagger-ui.css',
            swagger_favicon_url='/static/favicon-32x32.png',
        )



from __future__ import annotations

from pathlib import Path
from typing import Type

from pydantic import BaseModel

from .config_loader import ConfigLoader
from artemis_common.consts import artemis_env


CONFIG_DIR = Path(__file__).parent


def get_config(config_dir: Path, config_model: Type[BaseModel]) -> Type[BaseModel]:
    config_file_path = config_dir.joinpath('files', f'config_{artemis_env}.json')
    return ConfigLoader(config_file_path, config_model).config


def read_pem():
    pem_file = CONFIG_DIR.joinpath('pem.key')
    return pem_file.read_text()



from __future__ import annotations

from typing import Optional


class ArtemisException(Exception):
    pass


class ArtemisOperationFailedError(ArtemisException):
    pass


class ArtemisOperationUnexpectedlyFailedError(ArtemisException):
    def __init__(self, message: Optional[str] = None):
        if not message:
            message = ''
        super().__init__(f'[Unexpected] {message}')


class ArtemisUnexpectedFlowError(ArtemisException):
    def __init__(self):
        super().__init__('[Unexpected] Should not reach this line of code!')



from __future__ import annotations

from typing import Optional
from typing import Type

from pydantic import BaseModel
from pydantic import Field
from pydantic import root_validator


class ExceptionChain(BaseModel):
    type: str
    message: str


def get_exception_chain(
    exc: Type[Exception],
    exc_chain: list[ExceptionChain],
) -> list[ExceptionChain]:
    if exc.__cause__:
        cause = exc.__cause__
        exception_chain_item = ExceptionChain(
            type=type(cause).__name__,
            message=str(cause),
        )
        exc_chain.append(exception_chain_item)
        return get_exception_chain(cause, exc_chain)
    return exc_chain


class BaseServerResponseModel(BaseModel):
    status_code: int
    success: bool


class ServerSuccessResponseModel(BaseServerResponseModel):
    success: bool = Field(default=True)
    data: Optional[dict] = Field(default=None)


class ServerExceptionResponseModel(BaseServerResponseModel):
    class Config:
        arbitrary_types_allowed = True

    message: str
    error: str
    url: str
    path: str
    params: dict
    exc: Exception = Field(exclude=True)
    exc_chain: list[ExceptionChain] = Field(default=None)
    success: bool = Field(default=False)
    extra: Optional[dict] = Field(default=None)

    @root_validator(pre=True)
    def pre(cls, values: dict) -> dict:
        exc = values['exc']
        exc_chain = get_exception_chain(exc, [])
        values['exc_chain'] = exc_chain
        return values


__all__ = [
    BaseServerResponseModel.__name__,
    ServerSuccessResponseModel.__name__,
    ServerExceptionResponseModel.__name__,
]



from __future__ import annotations

from enum import Enum

from distutils.util import strtobool as str_to_bool
from pydantic import BaseSettings
from pydantic import Field
from pydantic import SecretStr


ARTEMIS_DAL_API_KEY_NAME = 'x-artemis-dal-api-key'


class ArtemisEnvironment(BaseSettings):
    artemis_env: str = 'dev'
    artemis_debug: str = 'False'


artemis_env = ArtemisEnvironment().artemis_env
is_prod_env = artemis_env == 'prod'
is_artemis_debug = str_to_bool(ArtemisEnvironment().artemis_debug)


class _EnvironmentVariables(BaseSettings):
    artemis_dal_api_key: SecretStr = Field(
        ...,
        env=f'{artemis_env}_ARTEMIS_DAL_API_KEY',
    )
